//Arduino code for the final project of module 2, Smart environments, by team 18 West Zakspeed Racing. 
//Smocore, a device that senses smoke, humidity and temperature in order to detect an upcoming wildfire and then sending a 
//message to the local authorities with the help of a SIM800L module, so action can be taken and the fire can be extinguished as quickly as possible.

//declare variables and add libraries
#include "SoftwareSerial.h"
#include <dht.h>
#define MQ2pin A0
#define DHTpin A1

float MQ2Value;
float DHTValue;

dht DHT;

SoftwareSerial mySerial(2, 3);

String cmd = "";

void setup()
{
  //setting correct baudrate
  mySerial.begin(9600);
  Serial.begin(9600);
  Serial.println("Initializing...");
  //Let the gas sensor warm up for 15 seconds so it can detect the particles in the air
  Serial.println("Gas sensor warming up!");
  delay(15000);

  mySerial.println("AT");                 // Sends an ATTENTION command, reply should be OK
  updateSerial();
  mySerial.println("AT+CMGF=1");          // Configuration for sending SMS
  updateSerial();
}

void loop()
{
  //read the sensor values and put them in the serial monitor
  MQ2Value = analogRead(MQ2pin);
  Serial.print("MQ2 sensor value: ");
  Serial.print(MQ2Value);
  DHTValue = analogRead(DHTpin);
  Serial.println(" ");
  Serial.print("Current humidity = ");
  Serial.print(DHT.humidity);
  Serial.print("%  ");
  Serial.print("temperature = ");
  Serial.print(DHT.temperature);
  Serial.println("C  ");

  if (MQ2Value > 700) {
    //if there are more than 700 ppm detected, send a warning message
    Serial.println(" ");
    Serial.print("Smoke/Gas detected!!");
  }
  Serial.println(" ");
  delay(50000);
  updateSerial(); //50 seconds delay between the reading
}

void updateSerial()
{
  delay(500);
  while (Serial.available())
  {

    cmd += (char)Serial.read();

    if (cmd != "") {
      cmd.trim();  // Remove added LF in transmit
      if (cmd.equals("S")) {
        sendSMS();
      } else {
        mySerial.print(cmd);
        mySerial.println("");
      }
    }
  }

  while (mySerial.available())
  {
    Serial.write(mySerial.read());//Forward what Software Serial received to Serial Port
  }
}

void sendSMS() {
  //check the connection with AT commands and send a message to the phone number
  mySerial.println("AT+CMGF=1");
  delay(500);
  mySerial.println("AT+CMGS=\"+0631544058\"\r");
  delay(500);
  mySerial.println("Initializing...");
  //read sensor values and text them
  mySerial.println("Gas sensor warming up!");
  mySerial.print("The MQ2-value =");
  mySerial.print(MQ2Value);
  mySerial.println(" ");
  mySerial.print("Current humidity = ");
  mySerial.print(DHT.humidity);
  mySerial.print("%  ");
  mySerial.print("Temperature = ");
  mySerial.print(DHT.temperature);
  mySerial.println("C  ");
  if (MQ2Value > 700) {
    mySerial.print("Smoke/Gas detected!!");
  }
  Serial.println(" ");
  delay(50000);
  updateSerial(); //50 seconds delay between the sending of the SMS
  mySerial.write(26);
}
